/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib;

import de.bsvrz.iav.fuzzylib.fuzzylib.FuzzySet;
import de.bsvrz.iav.fuzzylib.fuzzylib.LinguistischeVariable;

/**
 * Kapselt die Fuzzy-Sets und linguistische Variable sowie deren Testbelegung.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
@SuppressWarnings("nls")
public class ALkw implements TestDatumLV {

	/** Attributgruppe mit der linguistischen Variable. */
	public static final String ATG_FUZZY_SETS = "atg.anteilLkwFuzzy";

	/** Attributgruppe mit dem Fuzzy-Wert. */
	public static final String ATG_FUZZY_WERT = "atg.anteilLkw";

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsSehrGering;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsGering;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsMittel;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsHoch;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsSehrHoch;

	/**
	 * Linguistische Variable, die in mehreren Test verwendet wird. Sie wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public LinguistischeVariable lv;

	/**
	 * Testbelegung der Membervariablen erzeugen.
	 */
	public ALkw() {
		fsSehrGering = new FuzzySet("Sehr gering", 0, 0, 3, 7);
		fsGering = new FuzzySet("Gering", 3, 7, 8, 12);
		fsMittel = new FuzzySet("Mittel", 8, 12, 18, 22);
		fsHoch = new FuzzySet("Hoch", 18, 22, 23, 27);
		fsSehrHoch = new FuzzySet("Sehr hoch", 23, 27, 100, 100);

		lv = new LinguistischeVariable("ALkw", 0, 100);
		lv.addFuzzySet(fsSehrGering);
		lv.addFuzzySet(fsGering);
		lv.addFuzzySet(fsMittel);
		lv.addFuzzySet(fsHoch);
		lv.addFuzzySet(fsSehrHoch);
	}

	@Override
	public String getAtgFuzzySets() {
		return ATG_FUZZY_SETS;
	}

	@Override
	public String getAtgFuzzyWert() {
		return ATG_FUZZY_WERT;
	}

	@Override
	public LinguistischeVariable getLinguistischeVariable() {
		return new ALkw().lv;
	}

	@Override
	public LinguistischeVariable getLinguistischeVariable(final int szenario) {
		return new ALkw().lv;
	}

	@Override
	public String getName() {
		return getClass().getSimpleName();
	}

	@Override
	public String getSystemObjektTyp() {
		return "typ.messQuerschnittAllgemein";
	}

}
