/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib;

import de.bsvrz.iav.fuzzylib.fuzzylib.FuzzySet;
import de.bsvrz.iav.fuzzylib.fuzzylib.LinguistischeVariable;

/**
 * Kapselt die Fuzzy-Sets und linguistische Variable sowie deren Testbelegung.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
@SuppressWarnings("nls")
public class Helligkeit implements TestDatumLV {

	/** Attributgruppe mit der linguistischen Variable. */
	public static final String ATG_FUZZY_SETS = "atg.ufdsParameterHelligkeitFuzzy";

	/** Attributgruppe mit dem Fuzzy-Wert. */
	public static final String ATG_FUZZY_WERT = "atg.ufdsHelligkeitFuzzy";

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsDunkel;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsDaemmrig;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsHell;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsSehrHell;

	/**
	 * Linguistische Variable, die in mehreren Test verwendet wird. Sie wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 * <p>
	 * Windrichtung als linguistische Variable, Beispiel f&uuml;r den
	 * Sonderfall, dass ein Fuzzy-Set &uuml;ber beide Wertebereichsgrenzen geht.
	 */
	public LinguistischeVariable lv;

	/**
	 * Testbelegung der Membervariablen erzeugen.
	 */
	public Helligkeit() {
		fsDunkel = new FuzzySet("Dunkel", 0, 0, 85, 110);
		fsDaemmrig = new FuzzySet("Dämmrig", 85, 100, 1800, 2100);
		fsHell = new FuzzySet("Hell", 1800, 2100, 14500, 15500);
		fsSehrHell = new FuzzySet("Sehr hell", 14500, 15500, 60000, 60000);

		lv = new LinguistischeVariable("Helligkeit", 0, 60000);
		lv.addFuzzySet(fsDunkel);
		lv.addFuzzySet(fsDaemmrig);
		lv.addFuzzySet(fsHell);
		lv.addFuzzySet(fsSehrHell);
	}

	@Override
	public String getAtgFuzzySets() {
		return ATG_FUZZY_SETS;
	}

	@Override
	public String getAtgFuzzyWert() {
		return ATG_FUZZY_WERT;
	}

	@Override
	public LinguistischeVariable getLinguistischeVariable() {
		return new Helligkeit().lv;
	}

	@Override
	public LinguistischeVariable getLinguistischeVariable(final int szenario) {
		return new Helligkeit().lv;
	}

	@Override
	public String getName() {
		return "Helligkeit";
	}

	@Override
	public String getSystemObjektTyp() {
		return "typ.ufdsHelligkeit";
	}

}
