/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib;

import de.bsvrz.iav.fuzzylib.fuzzylib.FuzzySet;
import de.bsvrz.iav.fuzzylib.fuzzylib.LinguistischeVariable;

/**
 * Kapselt die Fuzzy-Sets und linguistische Variable sowie deren Testbelegung.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
@SuppressWarnings("nls")
public class Ni implements TestDatumLV {

	/** Attributgruppe mit der linguistischen Variable. */
	public static final String ATG_FUZZY_SETS = "atg.ufdsParameterNiederschlagsIntensitätFuzzy";

	/** Attributgruppe mit dem Fuzzy-Wert. */
	public static final String ATG_FUZZY_WERT = "atg.ufdsNiederschlagsIntensitätFuzzy";

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsKeinNiederschlag;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsSchwacherNiederschlag;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsMittlererNiederschlag;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsStarkerNiederschlag;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsSehrStarkerNiederschlag;

	/**
	 * Linguistische Variable, die in mehreren Test verwendet wird. Sie wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 * <p>
	 * Windrichtung als linguistische Variable, Beispiel f&uuml;r den
	 * Sonderfall, dass ein Fuzzy-Set &uuml;ber beide Wertebereichsgrenzen geht.
	 */
	public LinguistischeVariable lv;

	/**
	 * Testbelegung der Membervariablen erzeugen.
	 */
	public Ni() {
		fsKeinNiederschlag = new FuzzySet("Kein Niederschlag", 0, 0, 0, 1);
		fsSchwacherNiederschlag = new FuzzySet("Schwacher Niederschlag", 0, 1,
				4, 6);
		fsMittlererNiederschlag = new FuzzySet("Mittlerer Niederschlag", 4, 6,
				8, 12);
		fsStarkerNiederschlag = new FuzzySet("Starker Niederschlag", 8, 12, 14,
				16);
		fsSehrStarkerNiederschlag = new FuzzySet("Sehr starker Niederschlag",
				14, 16, 200, 200);

		lv = new LinguistischeVariable("Niederschlag", 0, 200);
		lv.addFuzzySet(fsKeinNiederschlag);
		lv.addFuzzySet(fsSchwacherNiederschlag);
		lv.addFuzzySet(fsMittlererNiederschlag);
		lv.addFuzzySet(fsStarkerNiederschlag);
		lv.addFuzzySet(fsSehrStarkerNiederschlag);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getAtgFuzzySets() {
		return ATG_FUZZY_SETS;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getAtgFuzzyWert() {
		return ATG_FUZZY_WERT;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public LinguistischeVariable getLinguistischeVariable() {
		return new Ni().lv;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public LinguistischeVariable getLinguistischeVariable(final int szenario) {
		return new Ni().lv;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getName() {
		return "NiederschlagsIntensität";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getSystemObjektTyp() {
		return "typ.ufdsNiederschlagsIntensität";
	}

}
