/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib;

import de.bsvrz.iav.fuzzylib.fuzzylib.FuzzySet;
import de.bsvrz.iav.fuzzylib.fuzzylib.LinguistischeVariable;

/**
 * Kapselt die Fuzzy-Sets und linguistische Variable sowie deren Testbelegung.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
@SuppressWarnings("nls")
public class QKfz implements TestDatumLV {

	/** Attributgruppe mit der linguistischen Variable. */
	public static final String ATG_FUZZY_SETS = "atg.verkehrsStärkeStundeFuzzy";

	/** Attributgruppe mit dem Fuzzy-Wert. */
	public static final String ATG_FUZZY_WERT = "atg.verkehrsStärkeStunde";

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsSehrNiedrig;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsNiedrig;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsMittel;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsHoch;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsSehrHoch;

	/**
	 * Linguistische Variable, die in mehreren Test verwendet wird. Sie wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public LinguistischeVariable lv;

	/**
	 * Testbelegung der Membervariablen erzeugen.
	 */
	public QKfz() {
		this(1);
	}

	/**
	 * Testbelegung der Membervariablen erzeugen.
	 *
	 * @param szenario
	 *            Szenario 1 bis 3
	 */
	public QKfz(final int szenario) {
		switch (szenario) {
		case 1:
			fsSehrNiedrig = new FuzzySet("Sehr niedrig", 0, 0, 500, 700);
			fsNiedrig = new FuzzySet("Niedrig", 500, 700, 1000, 1400);
			fsMittel = new FuzzySet("Mittel", 1000, 1400, 2000, 2400);
			fsHoch = new FuzzySet("Hoch", 2000, 2400, 3000, 3400);
			fsSehrHoch = new FuzzySet("Sehr hoch", 3000, 3400, 100000000,
					100000000);
			break;
		case 2:
			fsSehrNiedrig = new FuzzySet("Sehr niedrig", 0, 0, 750, 1050);
			fsNiedrig = new FuzzySet("Niedrig", 750, 1050, 1500, 2100);
			fsMittel = new FuzzySet("Mittel", 1500, 2100, 3000, 3600);
			fsHoch = new FuzzySet("Hoch", 3000, 3600, 4500, 5100);
			fsSehrHoch = new FuzzySet("Sehr hoch", 4500, 5100, 100000000,
					100000000);
			break;
		case 3:
			fsSehrNiedrig = new FuzzySet("Sehr niedrig", 0, 0, 1000, 1400);
			fsNiedrig = new FuzzySet("Niedrig", 1000, 1400, 2000, 2800);
			fsMittel = new FuzzySet("Mittel", 2000, 2800, 4000, 4800);
			fsHoch = new FuzzySet("Hoch", 4000, 4800, 6000, 6800);
			fsSehrHoch = new FuzzySet("Sehr hoch", 6000, 6800, 100000000,
					100000000);
			break;
		default:
			throw new IllegalArgumentException();
		}

		lv = new LinguistischeVariable("QKfz", 0, 100000000);
		lv.addFuzzySet(fsSehrNiedrig);
		lv.addFuzzySet(fsNiedrig);
		lv.addFuzzySet(fsMittel);
		lv.addFuzzySet(fsHoch);
		lv.addFuzzySet(fsSehrHoch);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getAtgFuzzySets() {
		return ATG_FUZZY_SETS;
	}

	/**
	 * {@inheritDoc}
	 */

	@Override
	public String getAtgFuzzyWert() {
		return ATG_FUZZY_WERT;
	}

	/**
	 * {@inheritDoc}
	 */

	@Override
	public LinguistischeVariable getLinguistischeVariable() {
		return new QKfz(1).lv;
	}

	/**
	 * {@inheritDoc}
	 */

	@Override
	public LinguistischeVariable getLinguistischeVariable(final int szenario) {
		return new QKfz(szenario).lv;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getName() {
		return getClass().getSimpleName();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getSystemObjektTyp() {
		return "typ.messQuerschnittAllgemein";
	}

}
