/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib;

import java.util.List;

import de.bsvrz.iav.fuzzylib.fuzzylib.FuzzyVariable;
import de.bsvrz.iav.fuzzylib.fuzzylib.inferenz.Wissensbasis;

/**
 * Fasst alle Testdaten die eine Wissensbasis darstellen zusammen.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
@SuppressWarnings("nls")
public enum TestDatenWB implements TestDatumWB {

	/** Verkehrszustand. */
	Verkehrszustand("de.bsvrz.iav.fuzzylib.Verkehrszustand"),

	/** Unruhe im Verkehrsflussr. */
	Unruhe("de.bsvrz.iav.fuzzylib.Unruhe");

	/** Testdatum dieses {@code enum}. */
	private final TestDatumWB datum;

	/**
	 * Privater Konstruktor zum initialisieren des Testdatums.
	 *
	 * @param klasse
	 *            Voll qualifizierter Name einer Klasse vom Typ
	 *            {@code TestDatumLV}
	 */
	private TestDatenWB(final String klasse) {
		try {
			final Class<?> c = Class.forName(klasse);
			datum = (TestDatumWB) c.newInstance();
		} catch (final ClassNotFoundException e) {
			e.printStackTrace();
			throw new IllegalStateException();
		} catch (final InstantiationException e) {
			e.printStackTrace();
			throw new IllegalStateException();
		} catch (final IllegalAccessException e) {
			e.printStackTrace();
			throw new IllegalStateException();
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public List<FuzzyVariable> getVariablen() {
		return datum.getVariablen();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Wissensbasis getWissensbasis() {
		return datum.getWissensbasis();
	}

}
