/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib;

import static de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogikHandler.UND;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.iav.fuzzylib.fuzzylib.FuzzySet;
import de.bsvrz.iav.fuzzylib.fuzzylib.FuzzyVariable;
import de.bsvrz.iav.fuzzylib.fuzzylib.LinguistischeVariable;
import de.bsvrz.iav.fuzzylib.fuzzylib.inferenz.Regel;
import de.bsvrz.iav.fuzzylib.fuzzylib.inferenz.TermSymbol;
import de.bsvrz.iav.fuzzylib.fuzzylib.inferenz.Wissensbasis;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.OperationsSymbol;

/**
 * Kapselt die Wissensbasis und Fuzzy-Variable sowie deren Testbelegung.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
@SuppressWarnings("nls")
public class Unruhe implements TestDatumWB {

	/** Attributgruppe mit der linguistischen Variable. */
	public static final String ATG_FUZZY_SETS = null;

	/** Attributgruppe mit dem Fuzzy-Wert. */
	public static final String ATG_FUZZY_WERT = "atg.verkehrsZustandUnruhe";

	/**
	 * Wissensbasis, die w&auml;hrend mehrerer Test verwendet wird. Sie wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public Wissensbasis wb;

	/**
	 * Fuzzy-Variable, die w&auml;hrend mehrerer Test verwendet wird. Sie wird
	 * vor jedem Test mit identischen Werten initialisiert.
	 */
	public LinguistischeVariable lv;

	/**
	 * Testbelegung der Membervariablen erzeugen.
	 */
	public Unruhe() {
		Regel regel;
		Ausdruck praemisse;

		wb = new Wissensbasis();
		lv = new LinguistischeVariable("Unruhe im Verkehrsfluss");
		lv.addFuzzySet(new FuzzySet("Unruhe im Verkehr", 0, 0));
		lv.addFuzzySet(new FuzzySet("Keine Unruhe", 0, 0));

		// SKfz.hoch & QKfz.hoch -> Unruhe im Verkehr
		praemisse = new OperationsSymbol(UND, new TermSymbol("SKfz", "Hoch"),
				new TermSymbol("QKfz", "Hoch"));
		regel = new Regel(praemisse, lv.getName(), "Unruhe im Verkehr");
		wb.addRegel(regel);

		// SKfz.hoch & QKfz.sehrHoch -> Unruhe im Verkehr
		praemisse = new OperationsSymbol(UND, new TermSymbol("SKfz", "Hoch"),
				new TermSymbol("QKfz", "Sehr hoch"));
		regel = new Regel(praemisse, lv.getName(), "Unruhe im Verkehr");
		wb.addRegel(regel);

		// SKfz.hoch & QKfz.mittel -> keine Unruhe
		praemisse = new OperationsSymbol(UND, new TermSymbol("SKfz", "Hoch"),
				new TermSymbol("QKfz", "Mittel"));
		regel = new Regel(praemisse, lv.getName(), "Keine Unruhe");
		wb.addRegel(regel);

		// SKfz.hoch & QKfz.niedrig -> keine Unruhe
		praemisse = new OperationsSymbol(UND, new TermSymbol("SKfz", "Hoch"),
				new TermSymbol("QKfz", "Niedrig"));
		regel = new Regel(praemisse, lv.getName(), "Keine Unruhe");
		wb.addRegel(regel);

		// SKfz.hoch & QKfz.sehrNiedrig -> keine Unruhe
		praemisse = new OperationsSymbol(UND, new TermSymbol("SKfz", "Hoch"),
				new TermSymbol("QKfz", "Sehr niedrig"));
		regel = new Regel(praemisse, lv.getName(), "Keine Unruhe");
		wb.addRegel(regel);

		// SKfz.niedrig -> keine Unruhe
		praemisse = new TermSymbol("SKfz", "Niedrig");
		regel = new Regel(praemisse, lv.getName(), "Keine Unruhe");
		wb.addRegel(regel);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public List<FuzzyVariable> getVariablen() {
		final List<FuzzyVariable> liste = new ArrayList<FuzzyVariable>();

		liste.add(TestDatenLV.QKfz.getLinguistischeVariable().fuzzyfiziere());
		liste.add(TestDatenLV.SKfz.getLinguistischeVariable().fuzzyfiziere());
		liste.add(lv.fuzzyfiziere());

		return liste;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Wissensbasis getWissensbasis() {
		return wb;
	}

}
