/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib;

import de.bsvrz.iav.fuzzylib.fuzzylib.FuzzySet;
import de.bsvrz.iav.fuzzylib.fuzzylib.LinguistischeVariable;

/**
 * Kapselt die Fuzzy-Sets und linguistische Variable sowie deren Testbelegung.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
@SuppressWarnings("nls")
public class WG implements TestDatumLV {

	/** Attributgruppe mit der linguistischen Variable. */
	public static final String ATG_FUZZY_SETS = "atg.ufdsParameterWindGeschwindigkeitMittelWertFuzzy";

	/** Attributgruppe mit dem Fuzzy-Wert. */
	public static final String ATG_FUZZY_WERT = "atg.ufdsWindGeschwindigkeitMittelWertFuzzy";

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsKeinWind;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsSchwacherWind;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsMittlererWind;

	/**
	 * Fuzzy-Set, was w&auml;hrend der mehrerer Test verwendet wird. Es wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 */
	public FuzzySet fsStarkerWind;

	/**
	 * Linguistische Variable, die in mehreren Test verwendet wird. Sie wird vor
	 * jedem Test mit identischen Werten initialisiert.
	 * <p>
	 * Windrichtung als linguistische Variable, Beispiel f&uuml;r den
	 * Sonderfall, dass ein Fuzzy-Set &uuml;ber beide Wertebereichsgrenzen geht.
	 */
	public LinguistischeVariable lv;

	/**
	 * Testbelegung der Membervariablen erzeugen.
	 */
	public WG() {
		fsKeinWind = new FuzzySet("Kein Wind", 0, 0, 0, 1);
		fsSchwacherWind = new FuzzySet("Schwacher Wind", 0, 1, 3, 4);
		fsMittlererWind = new FuzzySet("Mittlerer Wind", 3, 4, 8, 10);
		fsStarkerWind = new FuzzySet("Starker Wind", 8, 10, 100, 100);

		lv = new LinguistischeVariable("Windgeschwindigkeit", 0, 100);
		lv.addFuzzySet(fsKeinWind);
		lv.addFuzzySet(fsSchwacherWind);
		lv.addFuzzySet(fsMittlererWind);
		lv.addFuzzySet(fsStarkerWind);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getAtgFuzzySets() {
		return ATG_FUZZY_SETS;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getAtgFuzzyWert() {
		return ATG_FUZZY_WERT;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public LinguistischeVariable getLinguistischeVariable() {
		return new WG().lv;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public LinguistischeVariable getLinguistischeVariable(final int szenario) {
		return new WG().lv;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getName() {
		return "WindGeschwindigkeitMittelWert";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getSystemObjektTyp() {
		return "typ.ufdsWindGeschwindigkeitMittelWert";
	}

}
