/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib.fuzzylib;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.NoSuchElementException;

import org.junit.Test;

import de.bsvrz.iav.fuzzylib.KB;
import de.bsvrz.iav.fuzzylib.QKfz;

/**
 * Testet &ouml;ffentliche und interne Funktionen der Fuzzy-Variable.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
@SuppressWarnings("nls")
public class TestFuzzyVariable {

	/**
	 * Funktionstest f&uuml;r {@code equals()}.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void equals() {
		final KB kb = new KB(2);
		final FuzzyVariable fvKB1 = kb.lv.fuzzyfiziere(52);
		final FuzzyVariable fvKB2 = kb.lv.fuzzyfiziere(232);

		assertEquals(fvKB1, fvKB1);
		assertEquals(fvKB1, fvKB2);
		assertEquals(fvKB2, fvKB1);

		assertFalse(fvKB1.equals(""));
		assertFalse(fvKB1.equals(345));

		fvKB1.setName("besser als Test");
		assertEquals(fvKB1, fvKB1);
		assertFalse(fvKB1.equals(fvKB2));
		assertFalse(fvKB2.equals(fvKB1));

		fvKB2.setName("besser als Test");
		assertEquals(fvKB2, fvKB2);
		assertEquals(fvKB1, fvKB2);
		assertEquals(fvKB2, fvKB1);
	}

	/**
	 * Testet auf Zugriff nicht existenter Terme.
	 */
	@Test(expected = NoSuchElementException.class)
	@SuppressWarnings("static-method")
	public void getZugehoerigkeitFalscherTerm() {
		final QKfz qKfz = new QKfz(3);
		final FuzzyVariable fvQKfz = qKfz.lv.fuzzyfiziere(100);

		fvQKfz.getZugehoerigkeit("kalt");
	}

	/**
	 * Setzen und Auslesen des Namen.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void name() {
		final QKfz qKfz = new QKfz(2);
		final FuzzyVariable fvQKfz = qKfz.lv.fuzzyfiziere(52);

		assertEquals("QKfz", fvQKfz.getName());

		fvQKfz.setName("besser als Test");
		assertEquals("besser als Test", fvQKfz.getName());

		fvQKfz.setName(null);
		assertEquals("QKfz", fvQKfz.getName());
	}

	/**
	 * Unerlaubter Name.
	 */
	@Test(expected = IllegalArgumentException.class)
	@SuppressWarnings("static-method")
	public void nameLeerzeichen() {
		final QKfz qKfz = new QKfz(2);
		final FuzzyVariable fvQKfz = qKfz.lv.fuzzyfiziere(52);

		fvQKfz.setName("   ");
	}

	/**
	 * &Ouml;ffentlicher Konstruktor testen.
	 */
	@Test(expected = IllegalArgumentException.class)
	@SuppressWarnings("static-method")
	public void publicKonstruktorAusnahme() {
		@SuppressWarnings("unused")
		final FuzzyVariable v = new FuzzyVariable("egal");
	}

	/**
	 * Testet das Setzen von korrekten und falschen Werten f&uuml;r die
	 * Zugeh&ouml;rigkeit.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void setzenZugehoerigkeit() {
		final QKfz qKfz = new QKfz(3);
		final FuzzyVariable fvQKfz = qKfz.lv.fuzzyfiziere();

		for (float f = -0.5f; f <= 1.5; f += 0.1) {
			try {
				fvQKfz.setTerm(f);
				assertTrue((0 <= f) && (f <= 1));

				for (final Term t : fvQKfz.getTerme()) {
					assertEquals(f, t.get());
				}
			} catch (final IllegalArgumentException ex) {
				assertTrue((f < 0) || (f > 1));
			}
		}

		for (float f = -0.5f; f <= 1.5; f += 0.1) {
			try {
				for (final String term : fvQKfz.getTermnamen()) {
					fvQKfz.setTerm(term, f);
				}
				assertTrue((0 <= f) && (f <= 1));

				for (final Term t : fvQKfz.getTerme()) {
					assertEquals(f, t.get());
				}
			} catch (final IllegalArgumentException ex) {
				assertTrue((f < 0) || (f > 1));
			}
		}
	}

	/**
	 * Testet auf Zugriff nicht existenter Terme.
	 */
	@Test(expected = NoSuchElementException.class)
	@SuppressWarnings("static-method")
	public void setZugehoerigkeitFalscherTerm() {
		final QKfz qKfz = new QKfz(3);
		final FuzzyVariable fvQKfz = qKfz.lv.fuzzyfiziere(100);

		fvQKfz.setTerm("kalt", 0.0f);
	}

}
