/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib.fuzzylib;

import static org.junit.Assert.assertTrue;

import java.util.List;

import org.junit.Test;

import de.bsvrz.iav.fuzzylib.QKfz;
import de.bsvrz.iav.fuzzylib.VKfz;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;

/**
 * Testet die Klasse {@code Fuzzyfizierung}.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
@SuppressWarnings("nls")
public class TestFuzzyfizierung {

	/**
	 * F&uuml;hrt eine kleine Massenfuzzyfizierung durch.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void massenfuzzyfizierung() {
		final Fuzzyfizierung fuzzy = new Fuzzyfizierung();
		final Kontext kontext = new Kontext();
		final LinguistischeVariable vKfz2 = new VKfz(2).lv;
		final LinguistischeVariable qKfz2 = new QKfz(2).lv;
		FuzzyVariable fv;

		// Zu fuzzyfizierende Werte erfassen
		fuzzy.addBasisvariable("MQ1_VKfz", "VKfz2");
		fuzzy.addBasisvariable("MQ1_QKfz", "QKfz2");
		fuzzy.addBasisvariable("MQ2_VKfz", "VKfz2");
		fuzzy.addBasisvariable("MQ2_QKfz", "QKfz2");
		fuzzy.addBasisvariable("MQ3_VKfz", "VKfz2");
		fuzzy.addBasisvariable("MQ3_QKfz", "QKfz2");

		// Linguistische Variablen im Kontext ablegen
		kontext.set("VKfz2", vKfz2);
		kontext.set("QKfz2", qKfz2);

		// Messwerte im Kontext ablegen
		kontext.set("MQ1_VKfz", 80);
		kontext.set("MQ1_QKfz", 40);
		kontext.set("MQ2_VKfz", 35);
		kontext.set("MQ2_QKfz", 66);
		kontext.set("MQ3_VKfz", 98);
		kontext.set("MQ3_QKfz", 36);

		// Fuzzyfizieren
		final List<FuzzyVariable> erg = fuzzy.interpret(kontext);

		fv = vKfz2.fuzzyfiziere(80);
		fv.setName("MQ1_VKfz");
		assertTrue(erg.contains(fv));

		fv = qKfz2.fuzzyfiziere(40);
		fv.setName("MQ1_QKfz");
		assertTrue(erg.contains(fv));

		fv = vKfz2.fuzzyfiziere(35);
		fv.setName("MQ2_VKfz");
		assertTrue(erg.contains(fv));

		fv = qKfz2.fuzzyfiziere(66);
		fv.setName("MQ2_QKfz");
		assertTrue(erg.contains(fv));

		fv = vKfz2.fuzzyfiziere(98);
		fv.setName("MQ3_VKfz");
		assertTrue(erg.contains(fv));

		fv = qKfz2.fuzzyfiziere(36);
		fv.setName("MQ3_QKfz");
		assertTrue(erg.contains(fv));
	}

}
