/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib.fuzzylib;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

import de.bsvrz.iav.fuzzylib.VKfz;
import de.bsvrz.iav.fuzzylib.WR;

/**
 * Testet grundlegende Fuzzy-Funktionen, wie das Anlegen von linguistischen und
 * Fuzzy-Variablen sowie Vergleiche.
 *
 * Beim Ausf&uuml;hren des Test ist eine nicht 100%-ige &Uuml;berdeckung des
 * Quelltext normal, da es gew&uuml;nscht ist dass einige Bedingungen (
 * {@code assert}) und Ausnahmen ({@code Exception}) entweder immer auftreten
 * oder nie auftreten.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
@SuppressWarnings("nls")
public class TestLinguistischeVariable {

	/**
	 * Testvariable.
	 */
	private VKfz vKfz;

	/**
	 * Testvariable.
	 */
	private WR windrichtung;

	/**
	 * Absteigende Rampe.
	 */
	@Test
	public void absteigendeRampe() {
		FuzzyVariable fv;

		// Ecke t1 von Term "hoch"
		fv = vKfz.lv.fuzzyfiziere(vKfz.lv.getFuzzySet("Hoch").getT1());
		assertEquals(fv.getTerm("Niedrig").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Mittel").getZugehoerigkeit(), (Float) 1.0f);
		assertEquals(fv.getTerm("Hoch").getZugehoerigkeit(), (Float) 0.0f);

		// Ecke t2 von Term "hoch"
		fv = vKfz.lv.fuzzyfiziere(vKfz.lv.getFuzzySet("Hoch").getT2());
		assertEquals(fv.getTerm("Niedrig").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Mittel").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Hoch").getZugehoerigkeit(), (Float) 1.0f);

		// Ecke t3 von Term "hoch"
		fv = vKfz.lv.fuzzyfiziere(vKfz.lv.getFuzzySet("Hoch").getT3());
		assertEquals(fv.getTerm("Niedrig").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Mittel").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Hoch").getZugehoerigkeit(), (Float) 1.0f);

		// Ecke t4 von Term "hoch"
		fv = vKfz.lv.fuzzyfiziere(vKfz.lv.getFuzzySet("Hoch").getT4());
		assertEquals(fv.getTerm("Niedrig").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Mittel").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Hoch").getZugehoerigkeit(), (Float) 1.0f);
	}

	/**
	 * Aufsteigende Rampe.
	 */
	@Test
	public void aufsteigendeRampe() {
		FuzzyVariable fv;

		// Ecke t1 von Term "niedrig"
		fv = vKfz.lv.fuzzyfiziere(vKfz.lv.getFuzzySet("Niedrig").getT1());
		assertEquals(fv.getTerm("Niedrig").getZugehoerigkeit(), (Float) 1.0f);
		assertEquals(fv.getTerm("Mittel").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Hoch").getZugehoerigkeit(), (Float) 0.0f);

		// Ecke t2 von Term "niedrig"
		fv = vKfz.lv.fuzzyfiziere(vKfz.lv.getFuzzySet("Niedrig").getT2());
		assertEquals(fv.getTerm("Niedrig").getZugehoerigkeit(), (Float) 1.0f);
		assertEquals(fv.getTerm("Mittel").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Hoch").getZugehoerigkeit(), (Float) 0.0f);

		// Ecke t3 von Term "niedrig"
		fv = vKfz.lv.fuzzyfiziere(vKfz.lv.getFuzzySet("Niedrig").getT3());
		assertEquals(fv.getTerm("Niedrig").getZugehoerigkeit(), (Float) 1.0f);
		assertEquals(fv.getTerm("Mittel").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Hoch").getZugehoerigkeit(), (Float) 0.0f);

		// Ecke t4 von Term "niedrig"
		fv = vKfz.lv.fuzzyfiziere(vKfz.lv.getFuzzySet("Niedrig").getT4());
		assertEquals(fv.getTerm("Niedrig").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Mittel").getZugehoerigkeit(), (Float) 1.0f);
		assertEquals(fv.getTerm("Hoch").getZugehoerigkeit(), (Float) 0.0f);
	}

	/**
	 * Testet die Funktionen zum Pr&uuml;fen ob eine Definitionsl&uuml;cke
	 * existiert und zur Bestimmung dieser.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void definitionsLueckenTest1() {
		final LinguistischeVariable lv = new LinguistischeVariable("egal", 0,
				100);

		lv.addFuzzySet(new FuzzySet("niedrig", 0, 0, 30, 40));
		lv.addFuzzySet(new FuzzySet("mittel", 30, 40, 50, 60));
		lv.addFuzzySet(new FuzzySet("hoch", 50, 60, 100, 100));
		assertFalse(lv.hatLuecken());
		assertNull(lv.getLuecken());
	}

	/**
	 * Testet die Funktionen zum Pr&uuml;fen ob eine Definitionsl&uuml;cke
	 * existiert und zur Bestimmung dieser.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void definitionsLueckenTest2() {
		final LinguistischeVariable lv = new LinguistischeVariable("egal", 0,
				100);
		final List<Point2D> luecken = new ArrayList<Point2D>();

		lv.addFuzzySet(new FuzzySet("niedrig", 0, 0, 30, 40));
		lv.addFuzzySet(new FuzzySet("hoch", 50, 60, 100, 100));
		assertTrue(lv.hatLuecken());
		luecken.add(new Point2D.Double(40, 50));
		assertEquals(luecken, lv.getLuecken());
	}

	/**
	 * Testet die Funktionen zum Pr&uuml;fen ob eine Definitionsl&uuml;cke
	 * existiert und zur Bestimmung dieser.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void definitionsLueckenTest3() {
		final LinguistischeVariable lv = new LinguistischeVariable("egal", 0,
				100);
		final List<Point2D> luecken = new ArrayList<Point2D>();

		assertTrue(lv.hatLuecken());
		luecken.add(new Point2D.Double(0, 100));
		assertEquals(luecken, lv.getLuecken());
	}

	/**
	 * Testet die Funktionen zum Pr&uuml;fen ob eine Definitionsl&uuml;cke
	 * existiert und zur Bestimmung dieser.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void definitionsLueckenTest4() {
		final LinguistischeVariable lv = new LinguistischeVariable("egal", 0,
				100);

		lv.addFuzzySet(new FuzzySet("mittel", 0, 20, 80, 100));
		assertFalse(lv.hatLuecken());
		assertNull(lv.getLuecken());
	}

	/**
	 * Defuzzyfiziert einige Fuzzy-Sets.
	 */
	@Test
	public void defuzzyfiziere() {
		final FuzzyVariable fv = vKfz.lv.fuzzyfiziere();

		fv.setTerm("Niedrig", 0.0f);
		fv.setTerm("Mittel", 0.0f);
		fv.setTerm("Hoch", 0.0f);
		assertEquals(null, vKfz.lv.defuzzyfiziere(fv));

		fv.setTerm("Niedrig", 1.0f);
		fv.setTerm("Mittel", 0.0f);
		fv.setTerm("Hoch", 0.0f);
		assertEquals((Integer) 0, vKfz.lv.defuzzyfiziere(fv));

		fv.setTerm("Niedrig", 0.0f);
		fv.setTerm("Mittel", 1.0f);
		fv.setTerm("Hoch", 0.0f);
		assertEquals((Integer) 55, vKfz.lv.defuzzyfiziere(fv));

		fv.setTerm("Niedrig", 0.0f);
		fv.setTerm("Mittel", 0.0f);
		fv.setTerm("Hoch", 1.0f);
		assertEquals((Integer) 254, vKfz.lv.defuzzyfiziere(fv));

		fv.setTerm("Niedrig", 0.5f);
		fv.setTerm("Mittel", 0.5f);
		fv.setTerm("Hoch", 0.0f);
		assertEquals((Integer) 25, vKfz.lv.defuzzyfiziere(fv));

		fv.setTerm("Niedrig", 0.0f);
		fv.setTerm("Mittel", 0.5f);
		fv.setTerm("Hoch", 0.5f);
		assertEquals((Integer) 229, vKfz.lv.defuzzyfiziere(fv));

		fv.setTerm("Niedrig", 0.7f);
		fv.setTerm("Mittel", 0.4f);
		fv.setTerm("Hoch", 0.0f);
		assertEquals((Integer) 18, vKfz.lv.defuzzyfiziere(fv));
	}

	/**
	 * Defuzzyfiziert eine Tempertaurvariable mit der Geschwindigkeit.
	 */
	@Test(expected = IllegalArgumentException.class)
	public void falschesArgumentBeimDefuzzyfizieren1() {
		final LinguistischeVariable lv = new LinguistischeVariable("Temperatur",
				0, 100);
		lv.addFuzzySet(new FuzzySet("kalt", 0, 15));
		lv.addFuzzySet(new FuzzySet("warm", 20, 25));
		lv.addFuzzySet(new FuzzySet("heiß", 30, 100));

		final FuzzyVariable fv = lv.fuzzyfiziere(22);

		vKfz.lv.defuzzyfiziere(fv);
	}

	/**
	 * Defuzzyfiziert die Temperatur nachdem ein Term entfernt wurde.
	 */
	@Test(expected = IllegalArgumentException.class)
	@SuppressWarnings("static-method")
	public void falschesArgumentBeimDefuzzyfizieren2() {
		final LinguistischeVariable lv1 = new LinguistischeVariable(
				"Temperatur", 0, 100);
		lv1.addFuzzySet(new FuzzySet("kalt", 0, 15));
		lv1.addFuzzySet(new FuzzySet("warm", 20, 25));
		lv1.addFuzzySet(new FuzzySet("heiß", 30, 100));

		final LinguistischeVariable lv2 = new LinguistischeVariable(
				"Temperatur", 0, 100);
		lv2.addFuzzySet(new FuzzySet("kalt", 0, 15));
		lv2.addFuzzySet(new FuzzySet("heiß", 30, 100));

		final FuzzyVariable fv = lv1.fuzzyfiziere(22);

		lv2.defuzzyfiziere(fv);
	}

	/**
	 * Fuzzyfiziert mehrere Werte und pr&uuml;ft dabei insbesondere die
	 * Grenzwerte. Grenzwert sind hier die Trapezecken der Fuzzy-Sets. Es werden
	 * auch Werte dazwischen gepr&uuml;ft, ebenso der Spezialfall, wenn ein
	 * Fuzzy-Set die beiden Wertebereichsgrenzen &uuml;berspannt (z.B. Norden
	 * von 340° bis 20°).
	 */
	@Test
	public void fuzzyfizieren() {
		FuzzyVariable fv;
		final LinguistischeVariable lv = new LinguistischeVariable("egal");

		fv = lv.fuzzyfiziere();
		assertEquals(null, fv);

		fv = lv.fuzzyfiziere(15);
		assertEquals(null, fv);

		// Die Terme der linguistischen Variable müssen alle in die
		// Fuzzy-Variable übernommen werden
		fv = vKfz.lv.fuzzyfiziere();
		assertEquals(vKfz.lv.getTerme(), fv.getTermnamen());
	}

	/**
	 * Testet die Gleichheitsbedingungen.
	 */
	@Test
	public void gleichheit() {
		assertTrue(vKfz.lv.equals(vKfz.lv));

		assertFalse(vKfz.lv.equals(windrichtung.lv));
		assertFalse(vKfz.lv.equals("nix"));

		LinguistischeVariable lvVKfz = new VKfz(1).lv;
		lvVKfz.removeFuzzySet("Niedrig");
		assertFalse(vKfz.lv.equals(lvVKfz));

		lvVKfz = new VKfz(1).lv;
		lvVKfz.getFuzzySet("Niedrig").setT4(100);
		assertFalse(vKfz.lv.equals(lvVKfz));
	}

	/**
	 * Vor jedem Test werden die verwendeten Variablen mit einheitlichen
	 * Startwerten initialisiert.
	 */
	@Before
	public void initialisiereTest() {
		vKfz = new VKfz(1);
		windrichtung = new WR();
	}

	/**
	 * Pr&uuml;ft Bedingungen bei der Konstruktion von linguistischen Variablen.
	 */
	@Test
	public void linguistischeVariableKonstruktionNamen() {
		LinguistischeVariable lv;

		String name;
		final String[] richtig = { "Temperatur", "QKfz", "VKfz" };
		final String[] falsch = { "", " ", "\n", "\t" };

		// Standardwertebereich
		lv = new LinguistischeVariable(richtig[0]);
		assertEquals(0, lv.getMinimum());
		assertEquals(Integer.MAX_VALUE, lv.getMaximum());

		// Getter und Setter
		lv = new LinguistischeVariable(richtig[0], 10, 20);
		assertEquals(richtig[0], lv.getName());
		assertEquals(10, lv.getMinimum());
		assertEquals(20, lv.getMaximum());

		// Einfügen und Löschen
		lv = new LinguistischeVariable(richtig[0]);
		assertTrue(lv.addFuzzySet(vKfz.fsNiedrig));
		assertFalse(lv.addFuzzySet(vKfz.fsNiedrig));
		assertFalse(lv.addFuzzySet(vKfz.fsNiedrig));
		assertEquals(1, lv.getTerme().size());
		lv.removeFuzzySet("hoch");
		assertEquals(1, lv.getTerme().size());
		assertTrue(lv.addFuzzySet(vKfz.fsHoch));
		assertTrue(lv.addFuzzySet(vKfz.fsMittel));
		assertTrue(lv.replaceFuzzySet("Niedrig", vKfz.fsNiedrig));
		assertEquals(3, lv.getTerme().size());
		lv.removeFuzzySet("Niedrig");
		assertEquals(2, lv.getTerme().size());
		assertTrue(lv.addFuzzySet(vKfz.fsNiedrig));
		assertEquals(3, lv.getTerme().size());

		assertEquals(3, lv.getFuzzySet().size());
		assertTrue(lv.getFuzzySet().contains(vKfz.fsNiedrig));
		assertTrue(lv.getFuzzySet().contains(vKfz.fsMittel));
		assertTrue(lv.getFuzzySet().contains(vKfz.fsHoch));

		// Zuerst die richtigen, dann die falschen testen
		for (int i = 0; i < (richtig.length + falsch.length); i++) {
			if (i < richtig.length) {
				name = richtig[i];
			} else {
				name = falsch[i - richtig.length];
			}

			try {
				lv = new LinguistischeVariable(name);
				assertTrue(i < richtig.length);
			} catch (final IllegalArgumentException e) {
				assertTrue(i >= richtig.length);
			}
		}
	}

	/**
	 * Fuzzy-Set welches die Wertebereichsgrenzen &uuml;berspannt.
	 */
	@Test
	public void sonderfallDefuzzyfizieren() {
		final FuzzyVariable fv = windrichtung.lv.fuzzyfiziere();

		fv.setTerm(windrichtung.fsNorden.getName(), 0.0f);
		fv.setTerm(windrichtung.fsNordost.getName(), 0.0f);
		fv.setTerm(windrichtung.fsOsten.getName(), 0.0f);
		fv.setTerm(windrichtung.fsSuedost.getName(), 0.0f);
		fv.setTerm(windrichtung.fsSueden.getName(), 0.0f);
		fv.setTerm(windrichtung.fsSuedwest.getName(), 0.0f);
		fv.setTerm(windrichtung.fsWesten.getName(), 0.0f);
		fv.setTerm(windrichtung.fsNordwest.getName(), 0.0f);
		assertEquals(null, windrichtung.lv.defuzzyfiziere(fv));

		fv.setTerm(windrichtung.fsNorden.getName(), 0.0f);
		fv.setTerm(windrichtung.fsNordost.getName(), 0.0f);
		fv.setTerm(windrichtung.fsOsten.getName(), 0.0f);
		fv.setTerm(windrichtung.fsSuedost.getName(), 0.0f);
		fv.setTerm(windrichtung.fsSueden.getName(), 1.0f);
		fv.setTerm(windrichtung.fsSuedwest.getName(), 0.0f);
		fv.setTerm(windrichtung.fsWesten.getName(), 0.0f);
		fv.setTerm(windrichtung.fsNordwest.getName(), 0.0f);
		assertEquals((Integer) 180, windrichtung.lv.defuzzyfiziere(fv));

		fv.setTerm(windrichtung.fsNorden.getName(), 1.0f);
		fv.setTerm(windrichtung.fsNordost.getName(), 0.0f);
		fv.setTerm(windrichtung.fsOsten.getName(), 0.0f);
		fv.setTerm(windrichtung.fsSuedost.getName(), 0.0f);
		fv.setTerm(windrichtung.fsSueden.getName(), 0.0f);
		fv.setTerm(windrichtung.fsSuedwest.getName(), 0.0f);
		fv.setTerm(windrichtung.fsWesten.getName(), 0.0f);
		fv.setTerm(windrichtung.fsNordwest.getName(), 0.0f);
		assertEquals((Integer) 0, windrichtung.lv.defuzzyfiziere(fv));

		fv.setTerm(windrichtung.fsNorden.getName(), 0.4f);
		fv.setTerm(windrichtung.fsNordost.getName(), 0.8f);
		fv.setTerm(windrichtung.fsOsten.getName(), 0.0f);
		fv.setTerm(windrichtung.fsSuedost.getName(), 0.0f);
		fv.setTerm(windrichtung.fsSueden.getName(), 0.0f);
		fv.setTerm(windrichtung.fsSuedwest.getName(), 0.0f);
		fv.setTerm(windrichtung.fsWesten.getName(), 0.0f);
		fv.setTerm(windrichtung.fsNordwest.getName(), 0.2f);
		assertEquals(20.455, windrichtung.lv.defuzzyfiziere(fv), 1);
	}

	/**
	 * Fuzzy-Set welches die Wertebereichsgrenzen &uuml;berspannt.
	 */
	@Test
	public void sonderfallFuzzyfizieren() {
		FuzzyVariable fv;

		fv = windrichtung.lv.fuzzyfiziere(23);
		assertEquals(fv.getTerm("Norden").getZugehoerigkeit(), (Float) 0.4f);
		assertEquals(fv.getTerm("Nordost").getZugehoerigkeit(), (Float) 0.6f);
		assertEquals(fv.getTerm("Osten").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Südost").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Süden").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Südwest").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Westen").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Nordwest").getZugehoerigkeit(), (Float) 0.0f);

		fv = windrichtung.lv.fuzzyfiziere(0);
		assertEquals(fv.getTerm("Norden").getZugehoerigkeit(), (Float) 1.0f);
		assertEquals(fv.getTerm("Nordost").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Osten").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Südost").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Süden").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Südwest").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Westen").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Nordwest").getZugehoerigkeit(), (Float) 0.0f);

		fv = windrichtung.lv.fuzzyfiziere(359);
		assertEquals(fv.getTerm("Norden").getZugehoerigkeit(), (Float) 1.0f);
		assertEquals(fv.getTerm("Nordost").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Osten").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Südost").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Süden").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Südwest").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Westen").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Nordwest").getZugehoerigkeit(), (Float) 0.0f);

		fv = windrichtung.lv.fuzzyfiziere(360);
		assertEquals(fv.getTerm("Norden").getZugehoerigkeit(), (Float) 1.0f);
		assertEquals(fv.getTerm("Nordost").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Osten").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Südost").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Süden").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Südwest").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Westen").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Nordwest").getZugehoerigkeit(), (Float) 0.0f);

		fv = windrichtung.lv.fuzzyfiziere(-1);
		assertEquals(fv.getTerm("Norden").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Nordost").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Osten").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Südost").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Süden").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Südwest").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Westen").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Nordwest").getZugehoerigkeit(), (Float) 0.0f);
	}

	/**
	 * Trapez.
	 */
	@Test
	public void trapez() {
		FuzzyVariable fv;

		// Ecke t1 von Term "mittel"
		fv = vKfz.lv.fuzzyfiziere(vKfz.lv.getFuzzySet("Mittel").getT1());
		assertEquals(fv.getTerm("Niedrig").getZugehoerigkeit(), (Float) 1.0f);
		assertEquals(fv.getTerm("Mittel").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Hoch").getZugehoerigkeit(), (Float) 0.0f);

		// Ecke t2 von Term "mittel"
		fv = vKfz.lv.fuzzyfiziere(vKfz.lv.getFuzzySet("Mittel").getT2());
		assertEquals(fv.getTerm("Niedrig").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Mittel").getZugehoerigkeit(), (Float) 1.0f);
		assertEquals(fv.getTerm("Hoch").getZugehoerigkeit(), (Float) 0.0f);

		// Ecke t3 von Term "mittel"
		fv = vKfz.lv.fuzzyfiziere(vKfz.lv.getFuzzySet("Mittel").getT3());
		assertEquals(fv.getTerm("Niedrig").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Mittel").getZugehoerigkeit(), (Float) 1.0f);
		assertEquals(fv.getTerm("Hoch").getZugehoerigkeit(), (Float) 0.0f);

		// Ecke t4 von Term "mittel"
		fv = vKfz.lv.fuzzyfiziere(vKfz.lv.getFuzzySet("Mittel").getT4());
		assertEquals(fv.getTerm("Niedrig").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Mittel").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Hoch").getZugehoerigkeit(), (Float) 1.0f);

		fv = vKfz.lv.fuzzyfiziere(27);
		assertEquals(fv.getTerm("Niedrig").getZugehoerigkeit(), (Float) 0.8f);
		assertEquals(fv.getTerm("Mittel").getZugehoerigkeit(), (Float) 0.2f);
		assertEquals(fv.getTerm("Hoch").getZugehoerigkeit(), (Float) 0.0f);

		fv = vKfz.lv.fuzzyfiziere(82);
		assertEquals(fv.getTerm("Niedrig").getZugehoerigkeit(), (Float) 0.0f);
		assertEquals(fv.getTerm("Mittel").getZugehoerigkeit(), (Float) 0.3f);
		assertEquals(fv.getTerm("Hoch").getZugehoerigkeit(), (Float) 0.7f);
	}

}
