/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib.fuzzylib.aggregation;

import static de.bsvrz.iav.fuzzylib.interpreter.logik.FuzzyLogikHandler.MAXIMUM;
import static de.bsvrz.iav.fuzzylib.interpreter.logik.FuzzyLogikHandler.MINIMUM;
import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.List;

import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.iav.fuzzylib.VKfz;
import de.bsvrz.iav.fuzzylib.fuzzylib.FuzzyVariable;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.OperationsSymbol;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;

/**
 * Testet die Klasse {@code Aggregation}.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
@SuppressWarnings("nls")
public class TestAggregation {

	/**
	 * Registriert den Aggregationshandler.
	 */
	@BeforeClass
	public static void initialisieren() {
		Operator.registerHandler(new AggregationHandler());
	}

	/**
	 * Testet die Maximum-Operation.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void maximum() {
		final VKfz vkfz = new VKfz(1);
		final FuzzyVariable fv1 = vkfz.lv.fuzzyfiziere(50);
		final FuzzyVariable fv2 = vkfz.lv.fuzzyfiziere(80);
		final FuzzyVariable fv3 = vkfz.lv.fuzzyfiziere(120);

		final List<FuzzySymbol> fss = new ArrayList<FuzzySymbol>();
		fss.add(new FuzzySymbol("a"));
		fss.add(new FuzzySymbol("b"));
		fss.add(new FuzzySymbol("c"));

		final Kontext kontext = new Kontext();
		kontext.set("a", fv1);
		kontext.set("b", fv2);
		kontext.set("c", fv3);

		final Ausdruck a = new OperationsSymbol(MAXIMUM, fss);
		final Object obj = a.interpret(kontext);
		final FuzzyVariable agg = (FuzzyVariable) obj;

		assertEquals(0.0f, agg.getTerm("Niedrig").get());
		assertEquals(1.0f, agg.getTerm("Mittel").get());
		assertEquals(1.0f, agg.getTerm("Hoch").get());
	}

	/**
	 * Testet die Minimum-Operation.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void minimum() {
		final VKfz vkfz = new VKfz(1);
		final FuzzyVariable fv1 = vkfz.lv.fuzzyfiziere(30);
		final FuzzyVariable fv2 = vkfz.lv.fuzzyfiziere(50);
		final FuzzyVariable fv3 = vkfz.lv.fuzzyfiziere(32);

		final List<FuzzySymbol> fss = new ArrayList<FuzzySymbol>();
		fss.add(new FuzzySymbol("a"));
		fss.add(new FuzzySymbol("b"));
		fss.add(new FuzzySymbol("c"));

		final Kontext kontext = new Kontext();
		kontext.set("a", fv1);
		kontext.set("b", fv2);
		kontext.set("c", fv3);

		final Ausdruck a = new OperationsSymbol(MINIMUM, fss);
		final Object obj = a.interpret(kontext);
		final FuzzyVariable agg = (FuzzyVariable) obj;

		assertEquals(0.0f, agg.getTerm("Niedrig").get());
		assertEquals(0.5f, agg.getTerm("Mittel").get());
		assertEquals(0.0f, agg.getTerm("Hoch").get());
	}

}
