/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib.fuzzylib.inferenz;

import static de.bsvrz.iav.fuzzylib.interpreter.logik.FuzzyLogikHandler.ALGEBRAISCHE_SUMME;
import static org.junit.Assert.assertEquals;

import java.util.Map;

import org.junit.Before;
import org.junit.Test;

import de.bsvrz.iav.fuzzylib.KB;
import de.bsvrz.iav.fuzzylib.QKfz;
import de.bsvrz.iav.fuzzylib.SKfz;
import de.bsvrz.iav.fuzzylib.Unruhe;
import de.bsvrz.iav.fuzzylib.VKfz;
import de.bsvrz.iav.fuzzylib.Verkehrszustand;
import de.bsvrz.iav.fuzzylib.fuzzylib.FuzzyVariable;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;

/**
 * Erzeugt eine Wissensbasis und stellt mehrere Anfragen an diese.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
@SuppressWarnings("nls")
public class TestWissensbasis {

	/** Testbelegung. */
	private VKfz vKfz;

	/** Testbelegung. */
	private SKfz sKfz;

	/** Testbelegung. */
	private QKfz qKfz;

	/** Testbelegung. */
	private KB kb;

	/** Testbelegung. */
	private Verkehrszustand zustand;

	/** Testbelegung. */
	private Unruhe unruhe;

	/**
	 * Erster Test.
	 */
	@Test
	public void beispielA() {
		final Kontext kontext = new Kontext();

		FuzzyVariable fvVKfz;
		FuzzyVariable fvKB;
		Map<String, FuzzyVariable> folgerungen;
		FuzzyVariable fv;

		fvVKfz = vKfz.lv.fuzzyfiziere(78);
		kontext.set("VKfz", fvVKfz);

		fvKB = kb.lv.fuzzyfiziere(33);
		kontext.set("KB", fvKB);

		kontext.set("Verkehrszustand", zustand.lv.fuzzyfiziere());

		folgerungen = zustand.wb.interpret(kontext);

		assertEquals(1, folgerungen.size());
		fv = folgerungen.values().iterator().next();

		assertEquals((Float) 0.0f, fv.getTerm("Gestaut").getZugehoerigkeit());
		assertEquals((Float) 0.7f,
				fv.getTerm("Zähfließend").getZugehoerigkeit());
		assertEquals((Float) 0.3f, fv.getTerm("Dicht").getZugehoerigkeit());
		assertEquals((Float) 0.2f, fv.getTerm("Frei").getZugehoerigkeit());
	}

	/**
	 * Zweiter Test.
	 */
	@Test
	public void beispielB() {
		final Kontext kontext = new Kontext();

		FuzzyVariable fvVKfz;
		FuzzyVariable fvKB;
		Map<String, FuzzyVariable> folgerungen;
		FuzzyVariable fv;

		fvVKfz = vKfz.lv.fuzzyfiziere(26);
		kontext.set("VKfz", fvVKfz);

		fvKB = kb.lv.fuzzyfiziere(26);
		kontext.set("KB", fvKB);

		kontext.set("Verkehrszustand", zustand.lv.fuzzyfiziere());

		folgerungen = zustand.wb.interpret(kontext);

		assertEquals(1, folgerungen.size());
		fv = folgerungen.values().iterator().next();

		assertEquals((Float) 0.9f, fv.getTerm("Gestaut").getZugehoerigkeit());
		assertEquals((Float) 0.1f,
				fv.getTerm("Zähfließend").getZugehoerigkeit());
		assertEquals((Float) 0.0f, fv.getTerm("Dicht").getZugehoerigkeit());
		assertEquals((Float) 0.0f, fv.getTerm("Frei").getZugehoerigkeit());
	}

	/**
	 * Dritter Test.
	 */
	@Test
	public void beispielC() {
		final Kontext kontext = new Kontext();

		FuzzyVariable fvVKfz;
		FuzzyVariable fvKB;
		Map<String, FuzzyVariable> folgerungen;
		FuzzyVariable fv;

		fvVKfz = vKfz.lv.fuzzyfiziere(50);
		kontext.set("VKfz", fvVKfz);

		fvKB = kb.lv.fuzzyfiziere(40);
		kontext.set("KB", fvKB);

		kontext.set("Verkehrszustand", zustand.lv.fuzzyfiziere());

		folgerungen = zustand.wb.interpret(kontext);

		assertEquals(1, folgerungen.size());
		fv = folgerungen.values().iterator().next();

		assertEquals((Float) 0.0f, fv.getTerm("Gestaut").getZugehoerigkeit());
		assertEquals((Float) 1.0f,
				fv.getTerm("Zähfließend").getZugehoerigkeit());
		assertEquals((Float) 0.0f, fv.getTerm("Dicht").getZugehoerigkeit());
		assertEquals((Float) 0.0f, fv.getTerm("Frei").getZugehoerigkeit());
	}

	/**
	 * Sechster Test.
	 */
	@Test
	public void beispielD() {
		final Kontext kontext = new Kontext();

		FuzzyVariable fvQKfz;
		FuzzyVariable fvSKfz;
		Map<String, FuzzyVariable> folgerungen;
		FuzzyVariable fv;

		fvQKfz = qKfz.lv.fuzzyfiziere(2125);
		kontext.set("QKfz", fvQKfz);

		fvSKfz = sKfz.lv.fuzzyfiziere(30);
		kontext.set("SKfz", fvSKfz);

		kontext.set("Unruhe im Verkehrsfluss", unruhe.lv.fuzzyfiziere());

		folgerungen = unruhe.wb.interpret(kontext);

		assertEquals(1, folgerungen.size());
		fv = folgerungen.values().iterator().next();

		assertEquals((Float) 0.3125f,
				fv.getTerm("Unruhe im Verkehr").getZugehoerigkeit());
		assertEquals((Float) 0.6875f,
				fv.getTerm("Keine Unruhe").getZugehoerigkeit());
	}

	/**
	 * Siebenter Test.
	 */
	@Test
	public void beispielE() {
		final Kontext kontext = new Kontext();

		FuzzyVariable fvQKfz;
		FuzzyVariable fvSKfz;
		Map<String, FuzzyVariable> folgerungen;
		FuzzyVariable fv;

		fvQKfz = qKfz.lv.fuzzyfiziere(2125);
		kontext.set("QKfz", fvQKfz);

		fvSKfz = sKfz.lv.fuzzyfiziere(19);
		kontext.set("SKfz", fvSKfz);

		unruhe.wb.setOpAkkumulation(ALGEBRAISCHE_SUMME);
		assertEquals(ALGEBRAISCHE_SUMME, unruhe.wb.getOpAkkumulation());

		kontext.set("Unruhe im Verkehrsfluss", unruhe.lv.fuzzyfiziere());

		folgerungen = unruhe.wb.interpret(kontext);

		assertEquals(1, folgerungen.size());
		fv = folgerungen.values().iterator().next();

		assertEquals((Float) 0.25f,
				fv.getTerm("Unruhe im Verkehr").getZugehoerigkeit());
		assertEquals((Float) 0.8125f,
				fv.getTerm("Keine Unruhe").getZugehoerigkeit());
	}

	/**
	 * Vor jedem Test werden die verwendeten Variablen mit einheitlichen
	 * Startwerten initialisiert.
	 */
	@Before
	public void initialisiereTest() {
		vKfz = new VKfz(1);
		sKfz = new SKfz();
		qKfz = new QKfz(1);
		kb = new KB(1);
		zustand = new Verkehrszustand();
		unruhe = new Unruhe();
	}

}
