/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib.fuzzylib.modell;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.iav.fuzzylib.fuzzylib.FuzzyVariable;
import de.bsvrz.iav.fuzzylib.fuzzylib.LinguistischeVariable;
import de.bsvrz.sys.funclib.bitctrl.modell.AbstractOnlineDatensatz;
import de.bsvrz.sys.funclib.bitctrl.modell.Aspekt;
import de.bsvrz.sys.funclib.bitctrl.modell.Datum;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;

/**
 * Fasst gemeinsame Funktionen von Onlinedatens&auml;tzen zusammen, die
 * Fuzzy-Werte abbilden.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public abstract class AbstractOdFuzzyWert
		extends AbstractOnlineDatensatz<FuzzyWert>implements OdFuzzyWert {

	/**
	 * Initialisiert das Objekt.
	 *
	 * @param objekt
	 *            ein Systemobjekt.
	 */
	public AbstractOdFuzzyWert(final SystemObjekt objekt) {
		super(objekt);
	}

	/**
	 * Erstellt aus einem Datum eine Fuzzy-Variable.
	 *
	 * @param daten
	 *            ein Datum, welches eine Fuzzy-Variable sein muss.
	 * @return die Fuzzy-Variable.
	 */
	protected FuzzyVariable erstelleFV(final Data daten) {
		Array terme;
		List<String> termListe;
		FuzzyVariable fv;

		terme = daten.getItem(getName()).asArray();
		termListe = new ArrayList<String>();

		// Termnamen bestimmen
		if (terme.getLength() > 0) {
			for (int i = 0; i < terme.getLength(); i++) {
				termListe.add(terme.getItem(i).getTextValue("Name").getText()); //$NON-NLS-1$
			}
		}

		// Fuzzy-Variable anlegen
		fv = new FuzzyVariable(getName(),
				termListe.toArray(new String[termListe.size()]));

		// Zugehörigkeiten der Terme ergänzen
		if (terme.getLength() > 0) {
			for (int i = 0; i < terme.getLength(); i++) {
				Float z;
				final Data item = terme.getItem(i);
				final String term = item.getTextValue("Name").getText(); //$NON-NLS-1$

				if (item.getUnscaledValue("Zugehörigkeit").isState()) { //$NON-NLS-1$
					z = null;
				} else {
					z = item.getScaledValue("Zugehörigkeit").floatValue(); //$NON-NLS-1$
				}
				fv.setTerm(term, z);
			}
		}

		return fv;
	}

	@Override
	public FuzzyWert erzeugeDatum() {
		return new FuzzyWert();
	}

	@Override
	public Collection<Aspect> getAspekte() {
		final Set<Aspect> aspekte = new HashSet<Aspect>();
		for (final Aspekt a : Aspekte.values()) {
			aspekte.add(a.getAspekt());
		}
		return aspekte;
	}

	@Override
	protected Data konvertiere(final FuzzyWert datum) {
		LinguistischeVariable lv;
		FuzzyVariable fv;
		Data daten;
		Array terme;
		int i;

		daten = erzeugeSendeCache();
		fv = datum.getFuzzyWert();
		lv = getObjekt().getParameterDatensatz(getLinguistischeVariable())
				.getDatum().getLinguistischeVariable();
		terme = daten.getItem(getName()).asArray();
		terme.setLength(lv.getTerme().size());
		i = 0;
		for (final String term : lv.getTerme()) {
			terme.getItem(i).getTextValue("Name").setText(term); //$NON-NLS-1$
			if ((fv != null)
					&& (fv.getTerm(term).getZugehoerigkeit() != null)) {
				terme.getItem(i).getScaledValue("Zugehörigkeit") //$NON-NLS-1$
						.set(fv.getZugehoerigkeit(term));
			} else {
				terme.getItem(i).getUnscaledValue("Zugehörigkeit") //$NON-NLS-1$
						.setText("nicht ermittelbar"); //$NON-NLS-1$
			}
			i++;
		}

		return daten;
	}

	@Override
	public void setDaten(final ResultData result) {
		check(result);

		final FuzzyWert datum = new FuzzyWert();
		datum.setDatenStatus(Datum.Status.getStatus(result.getDataState()));
		if (result.hasData()) {
			datum.setZeitstempel(result.getDataTime());
			datum.setFuzzyWert(erstelleFV(result.getData()));
		}

		datum.setZeitstempel(result.getDataTime());
		setDatum(result.getDataDescription().getAspect(), datum);
		fireDatensatzAktualisiert(result.getDataDescription().getAspect(),
				datum.clone());
	}

}
