/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib.fuzzylib.modell;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.iav.fuzzylib.fuzzylib.FuzzySet;
import de.bsvrz.iav.fuzzylib.fuzzylib.LinguistischeVariable;
import de.bsvrz.sys.funclib.bitctrl.modell.AbstractParameterDatensatz;
import de.bsvrz.sys.funclib.bitctrl.modell.Datum;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;

/**
 * Fasst gemeinsame Funktionen von Parametern zusammen, die Fuzzy-Sets abbilden.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public abstract class AbstractPdFuzzySets
		extends AbstractParameterDatensatz<FuzzySets>implements PdFuzzySets {

	/**
	 * Ruft lediglich den Superkonstruktor auf.
	 *
	 * @param objekt
	 *            ein Systemobjekt.
	 */
	public AbstractPdFuzzySets(final SystemObjekt objekt) {
		super(objekt);
		setDatum(new FuzzySets());
	}

	/**
	 * Erstellt aus einem Datum eine linguistische Variable.
	 *
	 * @param daten
	 *            ein Datum, welches eine linguistische Variable sein muss.
	 * @return die linguistische Variable.
	 */
	protected LinguistischeVariable erstelleLV(final Data daten) {
		Array fuzzySets;
		LinguistischeVariable lv;

		lv = new LinguistischeVariable(getName());
		fuzzySets = daten.getItem(getName()).asArray();

		// Wertebereich
		lv.setMinimum(daten.getUnscaledValue("Minimum").intValue()); //$NON-NLS-1$
		lv.setMaximum(daten.getUnscaledValue("Maximum").intValue()); //$NON-NLS-1$

		// Fuzzy-Sets
		if (fuzzySets.getLength() > 0) {
			for (int i = 0; i < fuzzySets.getLength(); i++) {
				final Data item = fuzzySets.getItem(i);
				final String term = item.getTextValue("Name").getText(); //$NON-NLS-1$
				final int t1 = item.getUnscaledValue("t1").intValue(); //$NON-NLS-1$
				final int t2 = item.getUnscaledValue("t2").intValue(); //$NON-NLS-1$
				final int t3 = item.getUnscaledValue("t3").intValue(); //$NON-NLS-1$
				final int t4 = item.getUnscaledValue("t4").intValue(); //$NON-NLS-1$
				final FuzzySet fs = new FuzzySet(term, t1, t2, t3, t4);
				lv.addFuzzySet(fs);
			}
		}

		return lv;
	}

	@Override
	public FuzzySets erzeugeDatum() {
		return new FuzzySets();
	}

	@Override
	protected Data konvertiere(final FuzzySets datum) {
		final Data daten = erzeugeSendeCache();
		final LinguistischeVariable lv = datum.getLinguistischeVariable();
		int i = 0;
		Array fuzzySets;

		fuzzySets = daten.getItem(getName()).asArray();
		fuzzySets.setLength(lv.getFuzzySet().size());
		for (final FuzzySet fs : lv.getFuzzySet()) {
			fuzzySets.getItem(i).getTextValue("Name").setText(fs.getName()); //$NON-NLS-1$
			fuzzySets.getItem(i).getUnscaledValue("t1").set(fs.getT1()); //$NON-NLS-1$
			fuzzySets.getItem(i).getUnscaledValue("t2").set(fs.getT2()); //$NON-NLS-1$
			fuzzySets.getItem(i).getUnscaledValue("t3").set(fs.getT3()); //$NON-NLS-1$
			fuzzySets.getItem(i).getUnscaledValue("t4").set(fs.getT4()); //$NON-NLS-1$
			i++;
		}
		daten.getUnscaledValue("Minimum").set(lv.getMinimum()); //$NON-NLS-1$
		daten.getUnscaledValue("Maximum").set(lv.getMaximum()); //$NON-NLS-1$

		return daten;
	}

	@Override
	public void setDaten(final ResultData result) {
		check(result);

		final FuzzySets datum = new FuzzySets();
		datum.setDatenStatus(Datum.Status.getStatus(result.getDataState()));

		if (result.hasData()) {
			datum.setZeitstempel(result.getDataTime());
			datum.setLinguistischeVariable(erstelleLV(result.getData()));
			setDatum(datum);
			fireDatensatzAktualisiert(datum);
		}

		datum.setZeitstempel(result.getDataTime());
		setDatum(result.getDataDescription().getAspect(), datum);
		fireDatensatzAktualisiert(datum.clone());
	}

}
