/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib.fuzzylib.modell;

import de.bsvrz.iav.fuzzylib.fuzzylib.LinguistischeVariable;
import de.bsvrz.sys.funclib.bitctrl.modell.AbstractDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.Datum;

/**
 * Dieses Datum kapselt eine linguistische Variable und ihre Fuzzy-Sets.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public class FuzzySets extends AbstractDatum {

	/** Die linguistische Variable des Datum. */
	private LinguistischeVariable linguistischeVariable;

	/** aktueller Datenstatus. */
	private Status datenStatus = Datum.Status.UNDEFINIERT;

	@Override
	public FuzzySets clone() {
		final FuzzySets klon = new FuzzySets();
		klon.setZeitstempel(getZeitstempel());
		if (linguistischeVariable != null) {
			klon.linguistischeVariable = linguistischeVariable.clone();
		}
		klon.datenStatus = datenStatus;
		return klon;
	}

	@Override
	public Status getDatenStatus() {
		return datenStatus;
	}

	/**
	 * Gibt die linguistische Variable zur&uuml;ck.
	 *
	 * @return die linguistische Variable.
	 */
	public LinguistischeVariable getLinguistischeVariable() {
		return linguistischeVariable;
	}

	/**
	 * Legt die linguistische Variable fest.
	 *
	 * @param linguistischeVariable
	 *            die neue linguistische Variable.
	 */
	public void setLinguistischeVariable(
			final LinguistischeVariable linguistischeVariable) {
		this.linguistischeVariable = linguistischeVariable;
	}

	@Override
	public String toString() {
		String s;

		s = getClass().getName() + "["; //$NON-NLS-1$
		s += "zeitpunkt=" + getZeitpunkt(); //$NON-NLS-1$
		s += ", datenStatus=" + getDatenStatus(); //$NON-NLS-1$
		s += " , linguistischeVariable=" + linguistischeVariable; //$NON-NLS-1$
		s += "]"; //$NON-NLS-1$

		return s;
	}

	/**
	 * setzt den aktuzellen Datenstatus.
	 *
	 * @param datenStatus
	 *            der neue Status
	 */
	protected void setDatenStatus(final Status datenStatus) {
		this.datenStatus = datenStatus;
	}

}
