/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib.interpreter.logik;

import static de.bsvrz.iav.fuzzylib.interpreter.logik.FuzzyLogikHandler.ALGEBRAISCHES_PRODUKT;
import static de.bsvrz.iav.fuzzylib.interpreter.logik.FuzzyLogikHandler.ALGEBRAISCHE_SUMME;
import static de.bsvrz.iav.fuzzylib.interpreter.logik.FuzzyLogikHandler.GAMMA_OPERATOR;
import static de.bsvrz.iav.fuzzylib.interpreter.logik.FuzzyLogikHandler.KOMPLEMENT;
import static de.bsvrz.iav.fuzzylib.interpreter.logik.FuzzyLogikHandler.MAXIMUM;
import static de.bsvrz.iav.fuzzylib.interpreter.logik.FuzzyLogikHandler.MINIMUM;
import static de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogikHandler.IMPLIKATION;
import static de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogikHandler.NICHT;
import static de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogikHandler.ODER;
import static de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogikHandler.UND;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.List;

import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Handler;
import de.bsvrz.sys.funclib.bitctrl.interpreter.InterpreterException;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.OperationsSymbol;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischesLiteral;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischesSymbol;

/**
 * Testet die Interpretation logischer Ausdr&uuml;cke
 *
 * Beim Ausf&uuml;hren des Test ist eine nicht 100%-ige &Uuml;berdeckung des
 * Quelltext normal, da es gew&uuml;nscht ist dass einige Bedingungen (
 * {@code assert}) und Ausnahmen ({@code Exception}) entweder immer auftreten
 * oder nie auftreten.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
@SuppressWarnings("nls")
public class TestInterpreter {

	/**
	 * Registriert den notwenidgen Handler.
	 */
	@BeforeClass
	public static void registerHandler() {
		Operator.registerHandler(new FuzzyLogikHandler());
	}

	/**
	 * Pr&uuml;ft die Verkn&uuml;pfung mittels algebraischen Produkts.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void algebraischesProdukt() {
		Ausdruck ausdruck;
		Kontext kontext;

		ausdruck = new OperationsSymbol(ALGEBRAISCHES_PRODUKT,
				new LogischesSymbol("a"), new LogischesSymbol("b"));
		kontext = new Kontext();

		kontext.set("a", new LogischerWert(0.0f));
		kontext.set("b", new LogischerWert(0.0f));
		assertEquals(new LogischerWert(0.0f), ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(1.0f));
		kontext.set("b", new LogischerWert(1.0f));
		assertEquals(new LogischerWert(1.0f), ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(0.4f));
		kontext.set("b", new LogischerWert(0.5f));
		assertEquals(new LogischerWert(0.2f), ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(0.7f));
		kontext.set("b", new LogischerWert(0.1f));
		assertEquals(new LogischerWert(0.07f), ausdruck.interpret(kontext));
	}

	/**
	 * Pr&uuml;ft die Verkn&uuml;pfung mittels algebraischer Summe.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void algebraischeSumme() {
		Ausdruck ausdruck;
		Kontext kontext;

		ausdruck = new OperationsSymbol(ALGEBRAISCHE_SUMME,
				new LogischesSymbol("a"), new LogischesSymbol("b"));
		kontext = new Kontext();

		kontext.set("a", new LogischerWert(0.0f));
		kontext.set("b", new LogischerWert(0.0f));
		assertEquals(new LogischerWert(0.0f), ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(1.0f));
		kontext.set("b", new LogischerWert(1.0f));
		assertEquals(new LogischerWert(1.0f), ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(0.4f));
		kontext.set("b", new LogischerWert(0.5f));
		assertEquals(new LogischerWert(0.7f), ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(0.3f));
		kontext.set("b", new LogischerWert(0.8f));
		assertEquals(new LogischerWert(0.86f), ausdruck.interpret(kontext));
	}

	/**
	 * Testet allgemeine Funktionen, indem einige Ausdr&uuml;cke erstellt und
	 * ausgewertet werden.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void allgemein() {
		Ausdruck ausdruck;
		final List<LogischesLiteral> c = new ArrayList<LogischesLiteral>();

		// und
		ausdruck = new OperationsSymbol(UND, new LogischesLiteral(true),
				new LogischesLiteral(true));
		assertEquals(LogischerWert.WAHR, ausdruck.interpret(null));

		// oder
		c.add(new LogischesLiteral(true));
		c.add(new LogischesLiteral(true));
		c.add(new LogischesLiteral(false));
		c.add(new LogischesLiteral(false));
		ausdruck = new OperationsSymbol(ODER, c);
		assertEquals(LogischerWert.WAHR, ausdruck.interpret(null));

		// nicht
		ausdruck = new OperationsSymbol("nicht", new LogischesLiteral(true));
		assertEquals(LogischerWert.FALSCH, ausdruck.interpret(null));
	}

	/**
	 * Pr&uuml;ft die Verkn&uuml;pfung mittels Gamma-Operator.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void gammaOperator() {
		Ausdruck ausdruck;
		Kontext kontext;
		float fWert;

		ausdruck = new OperationsSymbol(GAMMA_OPERATOR,
				new LogischesSymbol("gamma"), new LogischesSymbol("a"),
				new LogischesSymbol("b"));
		kontext = new Kontext();

		// äquivalent zum algebraischen Produkt
		kontext.set("gamma", new LogischerWert(0.0f));
		kontext.set("a", new LogischerWert(0.5f));
		kontext.set("b", new LogischerWert(0.5f));
		assertEquals(new LogischerWert(0.25f), ausdruck.interpret(kontext));

		// äquivalent zur algebraischen Summe
		kontext.set("gamma", new LogischerWert(1.0f));
		kontext.set("a", new LogischerWert(0.5f));
		kontext.set("b", new LogischerWert(0.5f));
		assertEquals(new LogischerWert(0.75f), ausdruck.interpret(kontext));

		kontext.set("gamma", new LogischerWert(0.2f));
		kontext.set("a", new LogischerWert(0.9f));
		kontext.set("b", new LogischerWert(0.7f));
		fWert = (Float) ((LogischerWert) ausdruck.interpret(kontext)).get();
		assertEquals(0.69f, runde(fWert, 2), 0.00001);

		kontext.set("gamma", new LogischerWert(0.6f));
		kontext.set("a", new LogischerWert(0.3f));
		kontext.set("b", new LogischerWert(0.9f));
		fWert = (Float) ((LogischerWert) ausdruck.interpret(kontext)).get();
		fWert = runde(fWert, 2);
		assertEquals(0.57f, runde(fWert, 2), 0.00001);

		kontext.set("gamma", new LogischerWert(0.5f));
		kontext.set("a", new LogischerWert(1.0f));
		kontext.set("b", new LogischerWert(0.2f));
		fWert = (Float) ((LogischerWert) ausdruck.interpret(kontext)).get();
		assertEquals(0.45f, runde(fWert, 2), 0.00001);
	}

	/**
	 * Pr&uuml;ft die Bedingungen, ob der Logik-Handler angewandt werden kann.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void handlerSuche() {
		final Handler handler = new FuzzyLogikHandler();

		assertTrue(handler.validiereHandler(NICHT, new LogischerWert(false))
				.isValid());

		assertFalse(handler.validiereHandler(null, new LogischerWert(false))
				.isValid());
		assertFalse(handler.validiereHandler(UND).isValid());
		assertFalse(handler.validiereHandler(NICHT, 123).isValid());
		assertFalse(handler.validiereHandler(NICHT, true).isValid());
	}

	/**
	 * Pr&uuml;ft die Funktion der Implikation-Verkn&uuml;pfung.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void implikation() {
		Ausdruck ausdruck;
		Kontext kontext;

		ausdruck = new OperationsSymbol(IMPLIKATION, new LogischesSymbol("a"),
				new LogischesSymbol("b"));
		kontext = new Kontext();

		kontext.set("a", new LogischerWert(true));
		kontext.set("b", new LogischerWert(true));
		assertEquals(LogischerWert.WAHR, ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(true));
		kontext.set("b", new LogischerWert(false));
		assertEquals(LogischerWert.FALSCH, ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(false));
		kontext.set("b", new LogischerWert(true));
		assertEquals(LogischerWert.WAHR, ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(false));
		kontext.set("b", new LogischerWert(false));
		assertEquals(LogischerWert.WAHR, ausdruck.interpret(kontext));
	}

	/**
	 * Pr&uuml;ft die Funktion der Komplement-Verkn&uuml;pfung.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void komplement() {
		Ausdruck ausdruck;
		Kontext kontext;

		ausdruck = new OperationsSymbol(KOMPLEMENT, new LogischesSymbol("a"));
		kontext = new Kontext();

		kontext.set("a", new LogischerWert(0.0f));
		assertEquals(new LogischerWert(1.0f), ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(1.0f));
		assertEquals(new LogischerWert(0.0f), ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(0.5f));
		assertEquals(new LogischerWert(0.5f), ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(0.7f));
		assertEquals(new LogischerWert(0.3f), ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(0.2f));
		assertEquals(new LogischerWert(0.8f), ausdruck.interpret(kontext));
	}

	/**
	 * Pr&uuml;ft die Funktion der Maximum-Verkn&uuml;pfung.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void maximum() {
		Ausdruck ausdruck;
		Kontext kontext;

		ausdruck = new OperationsSymbol(MAXIMUM, new LogischesSymbol("a"),
				new LogischesSymbol("b"), new LogischesSymbol("c"),
				new LogischesSymbol("d"), new LogischesSymbol("e"),
				new LogischesSymbol("f"));
		kontext = new Kontext();

		kontext.set("a", new LogischerWert(0.5f));
		kontext.set("b", new LogischerWert(0.1f));
		kontext.set("c", new LogischerWert(0.7f));
		kontext.set("d", new LogischerWert(0.3f));
		kontext.set("e", new LogischerWert(0.9f));
		kontext.set("f", new LogischerWert(0.4f));
		assertEquals(new LogischerWert(0.9f), ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(1.0f));
		kontext.set("b", new LogischerWert(1.0f));
		kontext.set("c", new LogischerWert(1.0f));
		kontext.set("d", new LogischerWert(1.0f));
		kontext.set("e", new LogischerWert(1.0f));
		kontext.set("f", new LogischerWert(1.0f));
		assertEquals(new LogischerWert(1.0f), ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(0.0f));
		kontext.set("b", new LogischerWert(0.0f));
		kontext.set("c", new LogischerWert(0.0f));
		kontext.set("d", new LogischerWert(0.0f));
		kontext.set("e", new LogischerWert(0.0f));
		kontext.set("f", new LogischerWert(0.0f));
		assertEquals(new LogischerWert(0.0f), ausdruck.interpret(kontext));
	}

	/**
	 * Pr&uuml;ft die Funktion der Minimum-Verkn&uuml;pfung.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void minimum() {
		Ausdruck ausdruck;
		Kontext kontext;

		ausdruck = new OperationsSymbol(MINIMUM, new LogischesSymbol("a"),
				new LogischesSymbol("b"), new LogischesSymbol("c"),
				new LogischesSymbol("d"), new LogischesSymbol("e"),
				new LogischesSymbol("f"));
		kontext = new Kontext();

		kontext.set("a", new LogischerWert(0.5f));
		kontext.set("b", new LogischerWert(0.1f));
		kontext.set("c", new LogischerWert(0.7f));
		kontext.set("d", new LogischerWert(0.3f));
		kontext.set("e", new LogischerWert(0.9f));
		kontext.set("f", new LogischerWert(0.4f));
		assertEquals(new LogischerWert(0.1f), ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(0.0f));
		kontext.set("b", new LogischerWert(0.0f));
		kontext.set("c", new LogischerWert(0.0f));
		kontext.set("d", new LogischerWert(0.0f));
		kontext.set("e", new LogischerWert(0.0f));
		kontext.set("f", new LogischerWert(0.0f));
		assertEquals(new LogischerWert(0.0f), ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(1.0f));
		kontext.set("b", new LogischerWert(1.0f));
		kontext.set("c", new LogischerWert(1.0f));
		kontext.set("d", new LogischerWert(1.0f));
		kontext.set("e", new LogischerWert(1.0f));
		kontext.set("f", new LogischerWert(1.0f));
		assertEquals(new LogischerWert(1.0f), ausdruck.interpret(kontext));
	}

	/**
	 * Pr&uuml;ft die Funktion der Nicht-Verkn&uuml;pfung.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void nicht() {
		Ausdruck ausdruck;
		Kontext kontext;

		ausdruck = new OperationsSymbol(NICHT, new LogischesSymbol("a"));
		kontext = new Kontext();

		kontext.set("a", new LogischerWert(true));
		assertEquals(LogischerWert.FALSCH, ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(false));
		assertEquals(LogischerWert.WAHR, ausdruck.interpret(kontext));
	}

	/**
	 * Pr&uuml;ft die Funktion der Oder-Verkn&uuml;pfung.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void oder() {
		Ausdruck ausdruck;
		Kontext kontext;

		ausdruck = new OperationsSymbol(ODER, new LogischesSymbol("a"),
				new LogischesSymbol("b"));
		kontext = new Kontext();

		kontext.set("a", new LogischerWert(true));
		kontext.set("b", new LogischerWert(true));
		assertEquals(LogischerWert.WAHR, ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(true));
		kontext.set("b", new LogischerWert(false));
		assertEquals(LogischerWert.WAHR, ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(false));
		kontext.set("b", new LogischerWert(true));
		assertEquals(LogischerWert.WAHR, ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(false));
		kontext.set("b", new LogischerWert(false));
		assertEquals(LogischerWert.FALSCH, ausdruck.interpret(kontext));
	}

	/**
	 * Direktes Auswerten von Ausdr&uuml;cken durch den Handler.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void perform() {
		final Handler handler = new FuzzyLogikHandler();

		assertEquals(LogischerWert.FALSCH,
				handler.perform(NICHT, LogischerWert.WAHR));
	}

	/**
	 * {@code null} als Operator nicht erlaubt.
	 */
	@Test(expected = InterpreterException.class)
	@SuppressWarnings("static-method")
	public void performNull() {
		final Handler handler = new FuzzyLogikHandler();

		assertEquals(LogischerWert.FALSCH,
				handler.perform(null, LogischerWert.WAHR));
	}

	/**
	 * Pr&uuml;ft die Funktion der Und-Verkn&uuml;pfung.
	 */
	@Test
	@SuppressWarnings("static-method")
	public void und() {
		Ausdruck ausdruck;
		Kontext kontext;

		ausdruck = new OperationsSymbol(UND, new LogischesSymbol("a"),
				new LogischesSymbol("b"));
		kontext = new Kontext();

		kontext.set("a", new LogischerWert(true));
		kontext.set("b", new LogischerWert(true));
		assertEquals(LogischerWert.WAHR, ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(true));
		kontext.set("b", new LogischerWert(false));
		assertEquals(LogischerWert.FALSCH, ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(false));
		kontext.set("b", new LogischerWert(true));
		assertEquals(LogischerWert.FALSCH, ausdruck.interpret(kontext));

		kontext.set("a", new LogischerWert(false));
		kontext.set("b", new LogischerWert(false));
		assertEquals(LogischerWert.FALSCH, ausdruck.interpret(kontext));
	}

	/**
	 * Rundet eine Gleitkommazahl auf eine feste Anzahl Kommastellen.
	 *
	 * @param wert
	 *            Gleitkommazahl
	 * @param kommastellen
	 *            Anzahl gew&uuml;nschter Kommastellen
	 * @return Gerundete Gleitkommazahl
	 */

	private static float runde(final float wert, final int kommastellen) {
		long ergebnis;

		ergebnis = Math.round(wert * Math.pow(10, kommastellen));
		return (ergebnis
				/ Double.valueOf(Math.pow(10, kommastellen)).floatValue());
	}

}
